import FExt.EXT
from FList import LIST
from fairNLP import URL
from Core import HttpRequest, Soup, Extractor
from Core.Extractor import Extractor
from FLog.LOGGER import Log
import json
import sys
import timeout_decorator

Log = Log("FairWEB.ArchiveDownloader_v1")

class DownloadWebPage:
    response = None
    status = False
    result = False
    soup = None
    client = False
    # ->
    url = ""
    base_url = ""
    json = {}

    def __init__(self, url):
        sys.setrecursionlimit(10000)
        self.result = True
        self.url = url
        self.base_url = URL.extract_base_url(url)

    @classmethod
    def start_url(cls, url, client="False"):
        newExtractor = cls(url)
        newExtractor.url = url
        newExtractor.client = client
        newExtractor.request()
        if newExtractor.response:
            newExtractor.to_html()
            newExtractor.extract_data()
            print(json.dumps(newExtractor.json, indent=4, default=str))
            return newExtractor
        else:
            Log.i("Request was rejected by Server.")
            return False

    @classmethod
    def start_response(cls, url, response, client="False"):
        if not response:
            return False
        newExtractor = cls(url)
        newExtractor.response = response
        newExtractor.client = client
        newExtractor.to_html()
        newExtractor.extract_data()
        print(json.dumps(newExtractor.json, indent=4, default=str))
        return newExtractor

    def start_soup(self, soup):
        if not soup:
            Log.e("Soup is Empty or None.")
        self.soup = soup
        self.extract_data()
        # self.to_json()
        print(json.dumps(self.json, indent=4, default=str))

    # -> Step One -> Call URL and get Raw HTML back in Response Object.
    def request(self):
        Log.i(f"Making Request to URL = [ {self.url} ]")
        resp = HttpRequest.get_request(self.url)
        if resp:
            self.response = LIST.get(1, resp)

    # -> Step Two -> Convert Response Object to HTML Object
    def to_html(self):
        Log.i(f"Parsing Response Text to HTML Objects.")
        self.soup = Soup.Parse(self.response)

    # -> Step Four -> Extract Data from Elements/Tags
    @timeout_decorator.timeout(30)
    def extract_data(self):
        Log.i(f"Attempting to Extract Data from HTML Elements.")
        if not self.soup:
            return False
        try:
            temp = Extractor.Extract(self.soup, self.url, client=self.client)
            self.json = temp.data
            self.set_json("url", self.url)
            source = URL.get_site_name(self.url)
            self.set_json("client", self.client)
            self.set_json("source", source)
            self.set_json("source_url", f"www.{source}.com")
        except Exception as e:
            Log.e("Failed to Extract.", error=e)
            return False

    def set_json(self, key, value):
        self.json[key] = value

    def check_process(self):
        if not self.response:
            self.result = False
            return False
        if not self.soup:
            self.result = False
            return False
        return True

if __name__ == '__main__':
    newTest = "https://towardsdatascience.com/how-to-use-qgis-spatial-algorithms-with-python-scripts-4bf980e39898"  # denied
    newTest1 = "https://www.cnn.com/2022/03/27/politics/joe-biden-vladimir-putin-ukraine-war/index.html"  # denied
    newTest2 = "https://towardsdatascience.com/a-step-by-step-guide-to-scheduling-tasks-for-your-data-science-project-d7df4531fc41"
#     newTest3 = "https://www.americanbanker.com/payments/news/inside-ripples-plans-for-mainstream-crypto-payments"
    d = DownloadWebPage.start_url(url=newTest2)




from bs4 import BeautifulSoup
from bs4.element import Tag as bsTag
from Core import Tag
from FLog.LOGGER import Log
Log = Log("FairWeb.Core.Soup.Parse()")

# -> Step Two -> Convert Response Object to HTML Object
def to_html(response):
    return BeautifulSoup(response.text, 'html.parser')

def safe_find(tag, term):
    try:
        return tag.find(term)
    except Exception as e:
        Log.e("Failed to find term.", error=e)

def safe_findAll(tag, term):
    try:
        return tag.findAll(term)
    except Exception as e:
        Log.e("Failed to find term.", error=e)

class Parse:
    response = None
    status = False
    soup = None
    tag_body: bsTag = None
    tag_head: bsTag = None
    tag_time: bsTag = None
    element_img = None
    element_p1 = None
    tag_h1: bsTag = None
    element_meta = None
    element_span = None

    def __init__(self, response):
        self.response = response
        if self.to_html():
            self.extract_elements_and_tags()

    # -> Step Two -> Convert Response Object to HTML Object
    def to_html(self):
        if self.response:
            self.soup = BeautifulSoup(self.response.text, 'html.parser')
            return True
        else:
            return False

    # -> Step Three -> Convert HTML into Element/Tag Objects
    def extract_elements_and_tags(self):
        Log.i(f"Extracting HTML Elements.")
        self.tag_body = safe_find(self.soup, "body")  # -> 99%
        self.tag_head = safe_find(self.soup, "head")  # -> 99%
        self.tag_time = safe_find(self.tag_body, "time")  # ->
        self.element_img = safe_findAll(self.tag_body, "img")
        self.element_p1 = safe_findAll(self.soup, "p")  # ->
        self.tag_h1 = safe_find(self.tag_body, "h1")  # ->
        self.element_meta = safe_findAll(self.tag_head, "meta")
        self.element_span = safe_findAll(self.tag_body, "span")
        self.doTest()
        Log.i("Parsing Finished")

    def doTest(self):
        # test_tag =
        # test_tag = self.soup.find("meta", {"name": "keywords"})
        # test_attr = Tag.get_attribute(test_tag, "content")
        #
        # test_tag = self.soup.findAll("meta")
        # for item in test_tag:
        #     test = Tag.get_attribute(item, "name")
        #     if test and Regex.contains_any(["keywords"], test):
        #         keywords = Tag.get_attribute(item, "content")
        #         print(keywords)
        # test_attr = Tag.get_attribute(test_tag, "datetime")
        # print(test_attr)
        pass


from selenium import webdriver
from selenium.webdriver.common.keys import Keys

# driver = webdriver.Chrome('./chromedriver')
driver = webdriver.Chrome()
driver.get("https://public.totalglobalsports.com/public/event/2038/individual-team/18/11332/9")
print(driver.title)
# test = driver.find_elements(by=By.CLASS_NAME, value="table")
search_bar = driver.find_element_by_name("table")
print(search_bar)


from bs4.element import Tag as bsTag
from FSON import DICT
from FList import LIST
from fairNLP import Regex
from FLog.LOGGER import Log
Log = Log("FWEB.Tag")


"""
        1. Element -> holds Tag(s)
        2. Tag(s) -> are Nodes with a "next_element" that is another Tag(s).
        2. Tag(s) -> have key/value pairs.
        4. Tag(s) -> have "attrs" that is a dict of "attributes".

        Element = { Tag(s)? }
        Tag = { name, text, attrs?, next_element?/Tag? }
    """
def search(master_element, terms, enableName=True, enableText=True, enableAttributes=True):
    """ -> Master Search, Handles both Elements or Tags. <- """
    if not master_element:
        return False
    if not is_tag(master_element):
        # Element
        return search_element(master_element, terms, enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
    else:
        # Tag
        return search_tag_deep(master_element, terms,
                                 enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
def search_element(master_element, terms, enableName=False, enableText=False, enableAttributes=False):
    if not master_element:
        return False
    for tag in master_element:
        temp = find(tag, terms,
                        enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
        if temp:
            return temp
        result = search_tag_deep(tag, terms,
                                 enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
        if result:
            return result
    return False

def search_tag_deep(master_tag: bsTag, terms, enableName=False, enableText=False, enableAttributes=False):
    """ -> Loops All Elements in Tag, then Recursively Loops each Tag <- """
    if not master_tag:
        return False
    if not is_tag(master_tag):
        return False
    try:
        temp = find(master_tag, terms,
                        enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
        if temp:
            return temp
        tempMaster = search_tag(master_tag, terms,
                              enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
        if tempMaster:
            return tempMaster
        all_tags = master_tag.find_all_next()
        if all_tags:
            for subTag in all_tags:
                temp = search_tag(subTag, terms,
                                      enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
                if temp:
                    return temp
        Log.w(f" -> find_tag() found nothing for TERMS=[ {terms} ].")
        return False
    except Exception as e:
        Log.e(" -> find_tag() has failed.", error=e)
        return False

def search_tag(master_tag: bsTag, terms, enableName=True, enableText=True, enableAttributes=True, depth=0):
    """ -> A Recursive Loop through each "next_element"/Tag <- """
    if not is_tag(master_tag):
        return False
    try:
        Log.i(f"Searching... [ {master_tag.name} ]")
        # -> Search Tag
        result = find(master_tag, terms,
                          enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
        if result:
            return result
        else:
            count = depth + 1
            if count >= 2000:
                Log.i(f" -> search_tag() is still searching... Terms=[ {terms} ], Depth=[ {count} ]")
                return False
            next_tag = master_tag.next_element
            return search_tag(next_tag, terms,
                                  enableName=enableName, enableText=enableText, enableAttributes=enableAttributes,
                                  depth=count)
    except Exception as e:
        Log.e("Something went wrong during tag search.", error=e)
        return False

# def search_all_tag(master_tag: bsTag, terms, enableName=True, enableText=True, enableAttributes=True, depth=0) -> []:
#     """ -> EXPERIMENTAL AND NOT FINISHED <- """
#     if not is_tag(master_tag):
#         return False
#     try:
#         Log.i(f"Searching... [ {master_tag.name} ]")
#         results = []
#         # -> Search Tag
#         result = find(master_tag, terms,
#                           enableName=enableName, enableText=enableText, enableAttributes=enableAttributes)
#         if result:
#             results.append(result)
#             # return result
#         else:
#             count = depth + 1
#             if count >= 2000:
#                 Log.i(f" -> search_tag() is still searching... Terms=[ {terms} ]")
#                 return []
#             next_tag = master_tag.next_element
#             temp = search_tag(next_tag, terms,
#                                   enableName=enableName, enableText=enableText, enableAttributes=enableAttributes,
#                                   depth=count)
#             if temp:
#                 results.append(temp)
#         return results
#     except Exception as e:
#         Log.e("Something went wrong during tag search.", error=e)
#         return []

""" Next Layer """
def find(master_tag, terms, enableName, enableText, enableAttributes):
    """ -> Searches the single current  <- """
    Log.v(f" -> find() {terms}.")
    if not is_tag(master_tag):
        return False
    # -> Search Name
    if enableName:
        tagTemp = search_name(master_tag, terms)
        if tagTemp:
            return tagTemp
    # -> Search Text
    if enableText:
        text = search_text(master_tag, terms)
        if text:
            return text
    # -> Search Attributes
    if enableAttributes:
        attributes = search_attributes(master_tag, terms)
        if attributes:
            return attributes
        return False

def is_tag(master_tag):
    if not master_tag:
        Log.w(" -> search_tag() master_tag is Empty.")
        return False
    if not isinstance(master_tag, bsTag):
        Log.w(" -> search_tag() master_tag is not a [bsTag].")
        return False
    return True

def search_name(master_tag, terms):
    """ -> CORE SEARCH OF TAG NAME <- """
    if not is_tag(master_tag):
        return False
    tagName = master_tag.name
    tagTemp = Regex.contains_any(terms, content=tagName)
    if tagTemp:
        return "name", tagName, master_tag
    return False

def search_attributes(master_tag, terms):
    """ -> CORE SEARCH OF TAG ATTRIBUTES <- """
    if not is_tag(master_tag):
        return False
    try:
        attributes = DICT.get("attrs", master_tag)
        if attributes:
            for key in attributes.keys():
                value = DICT.get(key, attributes)
                kTemp = Regex.contains_any(terms, content=key)
                if kTemp:
                    Log.d(f" -> find() Key Found. KEY=[ {key} ].")
                    print("Key Found.", key)
                    return key, value, master_tag
                vTemp = Regex.contains_any(terms, content=value)
                if vTemp:
                    Log.d(f" -> find() Value Found. VALUE=[ {value} ].")
                    return key, value, master_tag
        return False
    except Exception as e:
        Log.e("Finding Attribute ", error=e)
        return False

def get_attribute(master_tag, key):
    """ -> CORE SEARCH OF TAG ATTRIBUTES <- """
    if not is_tag(master_tag):
        return False
    attributes = DICT.get("attrs", master_tag)
    if attributes:
        value = DICT.get(key, attributes)
        return value
    return False

def search_text(master_tag, terms):
    """ -> CORE SEARCH OF TAG TEXT <- """
    if not is_tag(master_tag):
        return False
    test = master_tag.text
    if test != "":
        tTemp = Regex.contains_any(terms, content=test)
        if tTemp:
            Log.d(f" -> find() Text Found. TEXT=[ {test} ].")
            return "text", master_tag.text, master_tag
    return False

def get_text(master_tag):
    """ -> CORE GET OF TAG TEXT <- """
    if not is_tag(master_tag):
        return False
    test = master_tag.text
    if test != "":
        Log.d(f" -> find() Text Found. TEXT=[ {test} ].")
        return master_tag.text
    return False

def get_value_for_key(master_tag, key):
    if not is_tag(master_tag):
        return False
    results = get_attribute(master_tag, key)
    if results:
        return results
    return False

def search_key(master_tag, terms):
    if not is_tag(master_tag):
        return False
    results = search_tag_deep(master_tag, terms, enableAttributes=True)
    if results:
        return LIST.get(0, results)
    return False

def search_value(master_tag, terms):
    if not is_tag(master_tag):
        return False
    results = search_tag_deep(master_tag, terms, enableAttributes=True)
    if results:
        return LIST.get(1, results)
    return False

def search_tag_name(master_tag, terms):
    if not is_tag(master_tag):
        return False
    results = search_tag_deep(master_tag, terms, enableName=True)
    if results:
        return results[2]
    return False

def contains_attribute(*attributes, tag):
    if not is_tag(tag):
        return False
    attributes = LIST.flatten(attributes)
    for attribute in attributes:
        if tag.has_attr(attribute):
            return attribute
    return False


import requests
import Resources
from FExt import EXT
from FLog.LOGGER import Log
Log = Log("FWEB.Core.HttpRequest")

HEADERS = {
    "scheme": "https",
    "method": "GET",
    "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36",
    "accept-language": "en-US",
    "accept-encoding": "gzip, deflate, br",
    "content-type": "text/html",
    "accept": "text/html",
    "referer": "https://www.google.com/",
}
HEADERS_EXT = {
    "scheme": "https",
    "path": "/public/event/2038/individual-team/18/11332/9",
    "method": "GET",
    "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36",
    "accept-language": "en-US,en;q=0.9",
    "accept-encoding": "gzip, deflate, br",
    "content-type": "text/html",
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
    "referer": "https://www.birminghamunited.com/",
    "sec-fetch-mode": "navigate",
    "sec-fetch-dest": "document",
    "sec-fetch-site": "cross-site",
    "sec-fetch-user": "?1",
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": "macOS",
    "upgrade-insecure-requests": "1",
    "sec-ch-ua": "'Not A;Brand';v='99', 'Chromium';v='99', 'Google Chrome';v='99'"
}
COOKIES = {'from-my': 'browser'}

PROXIES = {
  'http': 'http://10.10.1.10:3128',
  'https': 'http://10.10.1.10:1080',
}


# -> Step One -> Call URL and get Raw HTML back in Response Object.
# @Ext.safe_run
@EXT.sleep(5)
def get_request(url):
    try:
        HEADERS["user-agent"] = Resources.get_random_user_agent()
        Log.i("Making HTTP Request.", v=f"URL= [ {url} ] ")
        response = requests.get(url, headers=HEADERS, timeout=10)
        if response.status_code > 205:
            Log.w(f"Failed to make HTTP Request with URL= [ {url} ] ")
            return False
        else:
            Log.s(f"Successful HTTP Request")
            return True, response
    except Exception as e:
        Log.e("Request Failed.", error=e)
        return False



from Core import HttpRequest, Soup
from FLog.LOGGER import Log
Log = Log("FairWEB.Crawler.UrlExtractor")

def request_and_parse(_url):
    try:
        # Log.v("Making Request via requests and parsing html via bs4.")
        response = HttpRequest.get_request(_url)
        return Soup.to_html(response)
    except Exception as e:
        Log.e("Failed to make request.", error=e)

def extract_urls(url):
    Log.v("Extracting URLs via Soup.")
    return request_and_parse(url).findAll('a', href=True)


import os
from bs4 import BeautifulSoup
from FSON import DICT
from FList import LIST
from fairNLP import URL
from FusedDL import FusedDownloader
from Core import HttpRequest
from FQueue.UrlQueue import FQueue
from FLog.LOGGER import Log
Log = Log("FWEB.Crawler.ArchiveCrawler_v2")

way_back_machine_url = "https://web.archive.org"
avoid_list = ['youtube', 'twitter', 'facebook', 'instagram', 'advertising', 'apple', 'google',
              'spotify', 'tiktok', 'soundcloud', 'linkedin', 'flickr', 'oauth', 'terms-of-service',
              'privacy-policy', 'contact-us', 'fr', 'de', "coupon.", "coupons.", ".coupon", ".coupons"]


class SoupError(Exception):
    def __init__(self, message, errors=None):
        # Call the base class constructor with the parameters it needs
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors

class KillThySelf(Exception):
    def __init__(self, message, errors=None):
        # Call the base class constructor with the parameters it needs
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors

class Modes:
    NORMAL = "NORMAL"
    SUICIDE = "SUICIDE"
    KILL = "KILL"

class ArchiveCrawler:
    pid = None
    timer = None
    # -> kwargs
    max = 100
    suicideSwitch = False
    # -> Global
    MODE = Modes.NORMAL
    queue: FQueue = None
    extraction_success = []
    extraction_failed = []
    stay_within = ""
    soup = None
    total_count = 0

    def __init__(self, url, **kwargs):
        """
        -> Takes single URL, grabs every URL, then tries to download them all.
        -> Takes next URL, grabs every URL, tries to download them all.
        -> !Potential for Forever-Loop!
        """
        self.handle_kwargs(**kwargs)
        self.pid = os.getpid()
        self.stay_within = URL.get_site_name(url)
        self.queue = FQueue(maxSize=self.max, avoidList=avoid_list)
        Log.i(f"Launching ArchiveCrawler with PID=[ {self.pid} ]")
        self.queue.add(url)
        Log.i(f"Ready to START: ArchiveCrawler PID=[ {self.pid} ]")

    @classmethod
    def start(cls, _url, **kwargs):
        cls(_url, **kwargs).run()

    @classmethod
    def start_NormalMode(cls, _url, **kwargs):
        kwargs["suicideMode"] = False
        cls(_url, **kwargs).run()

    @classmethod
    def start_SuicideMode(cls, _url, **kwargs):
        kwargs["suicideMode"] = True
        cls(_url, **kwargs).run()

    def handle_kwargs(self, **kwargs):
        self.max = DICT.get("max", kwargs, default=100)
        self.suicideSwitch = DICT.get("suicideMode", kwargs, default=False)

    def run(self):
        Log.i(f"STARTING: ArchiveCrawler PID=[ {self.pid} ]")
        self.run_spider_queue()

    def verify_mode(self):
        if self.MODE == Modes.NORMAL:
            self.suicideCheck()
            return True
        if self.MODE == Modes.SUICIDE:
            return False
        if self.MODE == Modes.KILL:
            # -> If time goes beyond X amount in the future...
            self.killThySelf()

    def change_mode(self, newMode):
        self.MODE = newMode

    def run_spider_queue(self):
        Log.i(f"Starting URL Queue with {self.queue.size()}.")
        # -> Work Queue.
        while not self.queue.isEmpty():
            # -> Clean Queue Check.
            if self.queue.size() > self.max / 2:
                Log.v("Cleaning Queue.")
                self.queue.clean()
            Log.v("Getting next item in Queue.")
            _url = self.queue.get()
            # -> Attempt Download
            self.attempt_download(_url)
            # -> Begin to die? or Continue as Normal?
            if self.verify_mode():
                # -> Run URL Extraction
                self.extract_urls(_url)
            self.print_status()

    def attempt_download(self, url):
        Log.i("Attempting Archive Download.")
        result = FusedDownloader.download_v2(url)
        if result:
            Log.v("Extraction Success")
            self.extraction_success.append(url)
        else:
            Log.v("Extraction Failed")
            self.extraction_failed.append(url)

    def request(self, _url):
        try:
            Log.v("Making Request via requests and parsing html via bs4.")
            request_response = HttpRequest.get_request(_url)
            response = LIST.get(1, request_response)
            self.soup = BeautifulSoup(response.text, 'html.parser')
            Log.v("Request Made")
        except Exception as e:
            Log.e("Failed to make request.", error=e)

    def extract_urls(self, _url):
        # -> Request
        self.request(_url)
        # -> Parse out URLs
        if self.soup is None:
            return False
        Log.v("Extracting URLs via Soup.")
        extracted_urls = self.soup.findAll('a', href=True)
        # -> Add all URLs to Queue
        self.handle_extracted_urls(extracted_urls)

    """
    -> Need to verify that url we are getting isn't just the ext of the base url.
    -> We are missing URLS!!!
    url = www.cnet.com
    ext = '/news/steve-wilhite-creator-of-the-gif-dies/'
    """
    def handle_extracted_urls(self, extracted_urls):
        if not extracted_urls:
            Log.i("No Extracted URLS")
            return
        Log.i("Looping extracted urls.")
        for item in extracted_urls:
            if self.queue.isFull():
                Log.i("Queue is FULL!")
                return
            _url = DICT.get("href", item)
            Log.d(f"Looking at URL= [ {_url} ]")
            if str(_url).startswith("/web/"):
                Log.i(f"Url is in way back machine. Fixing url.. [ {_url} ].")
                _url = way_back_machine_url + _url
            if self.stay_within != "" and self.stay_within in _url:
                if _url and str(_url).startswith("http"):
                    self.queue.add(_url)
                else:
                    Log.v(f"Not inside staywithin [ {_url} ]")

    def suicideCheck(self):
        if self.suicideSwitch and self.queue.isFull():
            Log.i("Queue is FULL! Changing MODE to SUICIDE.")
            self.change_mode(Modes.SUICIDE)

    def killThySelf(self):
        raise KillThySelf("Kill Order Initiated.")

    def print_status(self):
        print("\n")
        Log.i(f"---------------------------------------------")
        # Log.i(f"Time: {DATE.to_hours_minutes_seconds(self.timer.current_time())} Hours:Minutes:Seconds.")
        Log.i(f"Spider Queue: {self.queue.size()}")
        Log.i(f"Extraction Success: {len(self.extraction_success)}")
        Log.i(f"Extraction Failed: {len(self.extraction_failed)}")
        Log.i(f"---------------------------------------------")
        print("\n")


if __name__ == '__main__':
    url = "https://web.archive.org/web/20130921072250/http://www.bloomberg.com/technology/"
    url2 = "https://web.archive.org/web/20131021173221/http://www.buzzfeed.com/"
    url3 = "https://www.nbcnews.com/"
    url_medium = "https://medium.com/"
    medium = "https://medium.com/"
    science = "https://www.science.org/"
    coindesk = 'https://www.coindesk.com/'
    cryptnews = "https://cryptonews.com/"
    coinbase = "https://blog.coinbase.com/"
    tokenEconomy = "https://tokeneconomy.co/archive"
    bbc = "https://www.bbc.com/"
    guardian = "https://www.theguardian.com/world/2022/jan/26/biden-threatens-putin-with-personal-sanctions-if-russia-invades-ukraine"
    marketwatch = "https://www.marketwatch.com/"
    imdb = "https://www.imdb.com/title/tt0110357/news?ref_=tt_ql_sm"
    cnet = "https://www.cnet.com/news/"
    verge = "https://www.theverge.com/tech"
    yahoo = "https://finance.yahoo.com/"
    engadget = "https://www.engadget.com/"
    # if guard in guardian:
    #     print(True)
    # c = ArchiveCrawler(engadget)



import random
import os

class Resource:
    USER_AGENTS = os.path.join(os.path.dirname(__file__), 'useragents.txt')

def get_random_user_agent():
    return get_random_resource(Resource.USER_AGENTS)

def get_random_resource(resource):
    """Uses generator to return next useragent in saved file
    """
    items = get_resource(resource)
    selection = random.randint(0, len(items) - 1)
    agent = items[selection]
    return agent.strip()

def get_resource(resource):
    """Uses generator to return next useragent in saved file
    """
    with open(resource, 'r') as f:
        agents = f.readlines()
        return agents


from newspaper import Article
from fairParser import JsonParser
from fairNLP import URL
from FLog.LOGGER import Log
Log = Log("FairWEB.newspaper3k.ArticleDownloader")

def download_article(url) -> bool:
    """
    - Attempts to download HTML/URL
    - Converts/Returns obj to HookupObj
    - Optional ability to validate hookupObj before returning.
    """
    article = Article(url)
    try:
        # -> Download HTML from URL
        article.download()
        # -> Initial Parsing from HTML to Article Object
        article.parse()
        Log.s(f"Successfully Downloaded Article. Attempting to Parse into Hookup.")
        # -> Parse from Article Object into Hookup Object
        json = JsonParser.parse(article, client="newspaper3k")
        json["source"] = URL.get_site_name(url)
        return json
    except Exception as e:
        Log.e("Failed to Download Article.", error=e)
        return False

if __name__ == '__main__':
    url1 = "https://cointelegraph.com/news/price-analysis-1-28-btc-eth-bnb-ada-sol-xrp-luna-doge-dot-avax"
    url2 = "https://www.forbes.com/sites/investor/2022/01/28/ethereum-cardano-polygon-solana-avalanche-and-polkadot-deathmatch-who-wins-who-dies/?sh=4292a4897c9b"
    url3 = "https://www.businessinsider.com/tomi-lahren-spoke-at-police-training-event-dismissed-reform-wapo-2022-1"
    url4 = "https://www.foxbusiness.com/business-leaders/elon-musk-offers-support-to-canadian-truckers-amid-covid-vaccine-mandate"
    newTest2 = "https://towardsdatascience.com/a-step-by-step-guide-to-scheduling-tasks-for-your-data-science-project-d7df4531fc41"
    download_article(url1)


from Downloader.ArchiveDownloader_v1 import DownloadWebPage
from Downloader import ArticleDownloader
from Core import Validator
from FExt import EXT
from FLog.LOGGER import Log
Log = Log("FairWEB.FusedDownloader")

@EXT.safe_run
def download_v1(url, saveToArchive=False, setDateToToday=False):
    # -> v1
    downloader_v1 = fweb_downloader_v1(url)
    if downloader_v1:
        result = Validator.validateAndSave(saveToArchive, setDateToToday, downloader_v1)
        return result
    # -> v2
    downloader_v2 = fweb_downloader_v2(url).json
    if downloader_v2:
        result = Validator.validateAndSave(saveToArchive, setDateToToday, downloader_v2)
        return result
    return False

@Validator.mongo_save
def download(url):
    return download_v2(url)

@Validator.mongo_save
def download_v2(url):
    # -> v1
    downloader_v1 = fweb_downloader_v1(url)
    if downloader_v1 and Validator.validate_article(downloader_v1):
        return downloader_v1
    # -> v2
    downloader_v2 = fweb_downloader_v2(url)
    if downloader_v2 and Validator.validate_article(downloader_v2):
        return downloader_v2.json
    return False

@Validator.mongo_save
def crawler_downloader(url, response):
    # -> v2
    downloader_v2 = fweb_response(url, response, client="archive crawler")
    if downloader_v2 and Validator.validate_article(downloader_v2):
        return downloader_v2.json
    # -> v1
    downloader_v1 = fweb_downloader_v1(url)
    if downloader_v1 and Validator.validate_article(downloader_v1):
        return downloader_v1
    return False

@Validator.mongo_save
def client_downloader(url, client):
    if not url:
        Log.e(f"Url argument is invalid.")
        return False
    # -> v1
    Log.i(f"1. Attempting newspaper3k Download of Article from CLIENT=[ {client} ]")
    downloader_v1 = fweb_downloader_v1(url)
    if downloader_v1:
        json_v1 = Validator.validate_article(downloader_v1)
        if json_v1:
            return downloader_v1
    # -> v2
    Log.i(f"2. Attempting FairDownloader Download of Article from CLIENT=[ {client} ]")
    downloader_v2 = fweb_downloader_v2(url, client=client)
    if downloader_v2:
        json_v2 = Validator.validate_article(downloader_v2)
        if json_v2:
            return downloader_v2.json
    return False

def fweb_downloader_v1(url):
    """ Original Article Downloader -> newspaper3k """
    return ArticleDownloader.download_article(url)

def fweb_downloader_v2(url, client="fweb_downloader_v2"):
    """ Fair Article Downloader -> URL Based """
    extractor = DownloadWebPage.start_url(url, client=client)
    return extractor

def fweb_response(url, response, client="fweb_response"):
    """ Fair Article Downloader -> HTTP Response Based """
    extractor = DownloadWebPage.start_response(url, response, client=client)
    return extractor

if __name__ == '__main__':
    url1 = "https://www.americanbanker.com/payments/news/inside-ripples-plans-for-mainstream-crypto-payments",
    url2 = "https://towardsdatascience.com/decorators-in-python-fundamentals-for-data-scientists-eada7f4eba85"
    url3 = "https://www.wsj.com/articles/meta-platforms-facebook-fb-q1-earnings-report-2022-1165102219"
    fweb_downloader_v2(url2)